@@ -517,7 +517,7 @@ void Qgs3DAxis::createAxisScene()
       if ( axisDirections.length() > 0 )
         mTextX->setText( QgsCoordinateReferenceSystemUtils::axisDirectionToAbbreviatedString( axisDirections.at( 0 ) ) );
       else
-        mTextY->setText( "X?" );
+        mTextX->setText( "X?" );
 
       if ( axisDirections.length() > 1 )
         mTextY->setText( QgsCoordinateReferenceSystemUtils::axisDirectionToAbbreviatedString( axisDirections.at( 1 ) ) );
