@@ -206,7 +206,8 @@ void QgsLayoutItemGroup::attemptResize( const QgsLayoutSize &size, bool includes
       command->saveBeforeState();
     }
 
-    QRectF itemRect = mapRectFromItem( item, item->rect() );
+    const QRectF originalItemRect = item->rect();
+    QRectF itemRect = mapRectFromItem( item, originalItemRect );
     QgsLayoutUtils::relativeResizeRect( itemRect, oldRect, newRect );
 
     itemRect = itemRect.normalized();
