@@ -194,8 +194,8 @@ bool QgsCoordinateTransformPrivate::initialize()
   ProjData res = threadLocalProjData();
 
 #ifdef COORDINATE_TRANSFORM_VERBOSE
-  QgsDebugMsg( "From proj : " + mSourceCRS.toProj() );
-  QgsDebugMsg( "To proj   : " + mDestCRS.toProj() );
+  QgsDebugMsgLevel( "From proj : " + mSourceCRS.toProj(), 2 );
+  QgsDebugMsgLevel( "To proj   : " + mDestCRS.toProj(), 2 );
 #endif
 
   if ( !res )
@@ -204,22 +204,20 @@ bool QgsCoordinateTransformPrivate::initialize()
 #ifdef COORDINATE_TRANSFORM_VERBOSE
   if ( mIsValid )
   {
-    QgsDebugMsg( QStringLiteral( "------------------------------------------------------------" ) );
-    QgsDebugMsg( QStringLiteral( "The OGR Coordinate transformation for this layer was set to" ) );
+    QgsDebugMsgLevel( QStringLiteral( "------------------------------------------------------------" ), 2 );
+    QgsDebugMsgLevel( QStringLiteral( "The OGR Coordinate transformation for this layer was set to" ), 2 );
     QgsLogger::debug<QgsCoordinateReferenceSystem>( "Input", mSourceCRS, __FILE__, __FUNCTION__, __LINE__ );
     QgsLogger::debug<QgsCoordinateReferenceSystem>( "Output", mDestCRS, __FILE__, __FUNCTION__, __LINE__ );
-    QgsDebugMsg( QStringLiteral( "------------------------------------------------------------" ) );
+    QgsDebugMsgLevel( QStringLiteral( "------------------------------------------------------------" ), 2 );
   }
   else
   {
-    QgsDebugMsg( QStringLiteral( "------------------------------------------------------------" ) );
-    QgsDebugMsg( QStringLiteral( "The OGR Coordinate transformation FAILED TO INITIALIZE!" ) );
-    QgsDebugMsg( QStringLiteral( "------------------------------------------------------------" ) );
+    QgsDebugError( QStringLiteral( "The OGR Coordinate transformation FAILED TO INITIALIZE!" ) );
   }
 #else
   if ( !mIsValid )
   {
-    QgsDebugMsg( QStringLiteral( "Coordinate transformation failed to initialize!" ) );
+    QgsDebugError( QStringLiteral( "Coordinate transformation failed to initialize!" ) );
   }
 #endif
 
@@ -246,27 +244,6 @@ void QgsCoordinateTransformPrivate::calculateTransforms( const QgsCoordinateTran
   }
 }
 
-static void proj_collecting_logger( void *user_data, int /*level*/, const char *message )
-{
-  QStringList *dest = reinterpret_cast< QStringList * >( user_data );
-  dest->append( QString( message ) );
-}
-
-static void proj_logger( void *, int level, const char *message )
-{
-#ifndef QGISDEBUG
-  Q_UNUSED( message )
-#endif
-  if ( level == PJ_LOG_ERROR )
-  {
-    QgsDebugMsg( QString( message ) );
-  }
-  else if ( level == PJ_LOG_DEBUG )
-  {
-    QgsDebugMsgLevel( QString( message ), 3 );
-  }
-}
-
 ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
 {
   QgsReadWriteLocker locker( mProjLock, QgsReadWriteLocker::Read );
@@ -284,8 +261,7 @@ ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
   locker.changeMode( QgsReadWriteLocker::Write );
 
   // use a temporary proj error collector
-  QStringList projErrors;
-  proj_log_func( context, &projErrors, proj_collecting_logger );
+  QgsScopedProjCollectingLogger errorLogger;
 
   mIsReversed = false;
 
@@ -335,7 +311,6 @@ ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
   {
     if ( !mSourceCRS.projObject() || ! mDestCRS.projObject() )
     {
-      proj_log_func( context, nullptr, nullptr );
       return nullptr;
     }
 
@@ -354,12 +329,17 @@ ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
       {
         // huh?
         const int errNo = proj_context_errno( context );
-        if ( errNo && errNo != -61 )
+        if ( errNo )
         {
+#if PROJ_VERSION_MAJOR>=8
+          nonAvailableError = QString( proj_context_errno_string( context, errNo ) );
+#else
           nonAvailableError = QString( proj_errno_string( errNo ) );
+#endif
         }
         else
         {
+          // in theory should never be hit!
           nonAvailableError = QObject::tr( "No coordinate operations are available between these two reference systems" );
         }
       }
@@ -487,9 +467,14 @@ ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
   if ( !transform && nonAvailableError.isEmpty() )
   {
     const int errNo = proj_context_errno( context );
-    if ( errNo && errNo != -61 )
+    const QStringList projErrors = errorLogger.errors();
+    if ( errNo )
     {
+#if PROJ_VERSION_MAJOR>=8
+      nonAvailableError = QString( proj_context_errno_string( context, errNo ) );
+#else
       nonAvailableError = QString( proj_errno_string( errNo ) );
+#endif
     }
     else if ( !projErrors.empty() )
     {
@@ -522,9 +507,6 @@ ProjData QgsCoordinateTransformPrivate::threadLocalProjData()
     }
   }
 
-  // reset logger to terminal output
-  proj_log_func( context, nullptr, proj_logger );
-
   if ( !transform )
   {
     // ouch!
