@@ -219,19 +219,19 @@ void QgsMapLayerServerProperties::copyTo( QgsMapLayerServerProperties *propertie
   QgsServerWmsDimensionProperties::copyTo( properties );
 }
 
-void QgsMapLayerServerProperties::reset()
+void QgsMapLayerServerProperties::reset() // cppcheck-suppress duplInheritedMember
 {
   QgsServerMetadataUrlProperties::reset();
   QgsServerWmsDimensionProperties::reset();
 }
 
-void QgsMapLayerServerProperties::readXml( const QDomNode &layer_node )
+void QgsMapLayerServerProperties::readXml( const QDomNode &layer_node ) // cppcheck-suppress duplInheritedMember
 {
   QgsServerMetadataUrlProperties::readXml( layer_node );
   QgsServerWmsDimensionProperties::readXml( layer_node );
 }
 
-void QgsMapLayerServerProperties::writeXml( QDomNode &layer_node, QDomDocument &document ) const
+void QgsMapLayerServerProperties::writeXml( QDomNode &layer_node, QDomDocument &document ) const // cppcheck-suppress duplInheritedMember
 {
   QgsServerMetadataUrlProperties::writeXml( layer_node, document );
   QgsServerWmsDimensionProperties::writeXml( layer_node, document );
