@@ -40,6 +40,8 @@
 #include <QCursor>
 #include <QPixmap>
 #include <QStatusBar>
+#include <algorithm>
+#include <memory>
 
 
 QgsMapToolCapture::QgsMapToolCapture( QgsMapCanvas *canvas, QgsAdvancedDigitizingDockWidget *cadDockWidget, CaptureMode mode )
@@ -314,7 +316,6 @@ bool QgsMapToolCapture::tracingAddVertex( const QgsPointXY &point )
   mSnappingMatches.removeLast();
   mSnappingMatches.append( QgsPointLocator::Match() );
 
-  int pointBefore = mCaptureCurve.numPoints();
   addCurve( new QgsLineString( mapPoints ) );
 
   resetRubberBand();
@@ -322,14 +323,18 @@ bool QgsMapToolCapture::tracingAddVertex( const QgsPointXY &point )
   // Curves de-approximation
   if ( QgsSettingsRegistryCore::settingsDigitizingConvertToCurve->value() )
   {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    int pointBefore = mCaptureCurve.numPoints();
+#endif
+
     // If the tool and the layer support curves
     QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer() );
     if ( vlayer && capabilities().testFlag( QgsMapToolCapture::Capability::SupportsCurves ) && vlayer->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::Capability::CircularGeometries ) )
     {
       const QgsGeometry linear = QgsGeometry( mCaptureCurve.segmentize() );
       const QgsGeometry curved = linear.convertToCurves(
-                                   QgsSettingsRegistryCore::settingsDigitizingConvertToCurveAngleTolerance->value(),
-                                   QgsSettingsRegistryCore::settingsDigitizingConvertToCurveDistanceTolerance->value()
+                                   QgsSettingsRegistryCore::settingsDigitizingConvertToCurveDistanceTolerance->value(),
+                                   QgsSettingsRegistryCore::settingsDigitizingConvertToCurveAngleTolerance->value()
                                  );
       if ( QgsWkbTypes::flatType( curved.wkbType() ) != Qgis::WkbType::CompoundCurve )
       {
@@ -341,12 +346,20 @@ bool QgsMapToolCapture::tracingAddVertex( const QgsPointXY &point )
         mCaptureCurve = *qgsgeometry_cast<QgsCompoundCurve *>( curved.constGet() );
       }
     }
-  }
 
-  // sync the snapping matches list
-  const int pointAfter = mCaptureCurve.numPoints();
-  for ( ; pointBefore < pointAfter; ++pointBefore )
-    mSnappingMatches.append( QgsPointLocator::Match() );
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    // sync the snapping matches list
+    const int pointAfter = mCaptureCurve.numPoints();
+
+    for ( ; pointBefore < pointAfter; ++pointBefore )
+      mSnappingMatches.append( QgsPointLocator::Match() );
+
+    for ( ; pointBefore > pointAfter; --pointBefore )
+      mSnappingMatches.removeLast();
+#else
+    mSnappingMatches.resize( mCaptureCurve.numPoints() );
+#endif
+  }
 
   tracer->reportError( QgsTracer::ErrNone, true ); // clear messagebar if there was any error
 
@@ -1381,21 +1394,42 @@ void QgsMapToolCapture::cadCanvasReleaseEvent( QgsMapMouseEvent *e )
     if ( digitizingFinished )
     {
       QgsGeometry g;
-      QgsCurve *curveToAdd = captureCurve()->clone();
+      std::unique_ptr<QgsCurve> curveToAdd( captureCurve()->clone() );
 
       if ( mode() == CaptureLine )
       {
-        g = QgsGeometry( curveToAdd );
+        g = QgsGeometry( curveToAdd->clone() );
         geometryCaptured( g );
-        lineCaptured( curveToAdd );
+        lineCaptured( curveToAdd.release() );
       }
       else
       {
-        QgsCurvePolygon *poly = new QgsCurvePolygon();
-        poly->setExteriorRing( curveToAdd );
-        g = QgsGeometry( poly );
+
+        //does compoundcurve contain circular strings?
+        //does provider support circular strings?
+        if ( QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer() ) )
+        {
+          const bool hasCurvedSegments = captureCurve()->hasCurvedSegments();
+          const bool providerSupportsCurvedSegments = vlayer->dataProvider()->capabilities() & QgsVectorDataProvider::CircularGeometries;
+
+          if ( hasCurvedSegments && providerSupportsCurvedSegments )
+          {
+            curveToAdd.reset( captureCurve()->clone() );
+          }
+          else
+          {
+            curveToAdd.reset( captureCurve()->curveToLine() );
+          }
+        }
+        else
+        {
+          curveToAdd.reset( captureCurve()->clone() );
+        }
+        std::unique_ptr<QgsCurvePolygon> poly{new QgsCurvePolygon()};
+        poly->setExteriorRing( curveToAdd.release() );
+        g = QgsGeometry( poly->clone() );
         geometryCaptured( g );
-        polygonCaptured( poly );
+        polygonCaptured( poly.get() );
       }
 
       stopCapturing();
