@@ -19,19 +19,50 @@ __author__ = 'Andrea Aime'
 __date__ = 'July 2016'
 __copyright__ = '(C) 2012, Andrea Aime'
 
+import os
+
 import qgis  # NOQA
-
-from qgis.PyQt.QtCore import Qt, QDir, QFile, QIODevice, QPointF, QSizeF
+from qgis.PyQt.QtCore import (
+    QDir,
+    QFile,
+    QIODevice,
+    QPointF,
+    QSizeF,
+    Qt,
+    QTemporaryDir,
+)
+from qgis.PyQt.QtGui import QColor, QFont, QImage
 from qgis.PyQt.QtXml import QDomDocument
-from qgis.PyQt.QtGui import QColor, QFont
-
 from qgis.core import (
-    QgsSimpleMarkerSymbolLayer, QgsSimpleMarkerSymbolLayerBase, QgsUnitTypes, QgsSvgMarkerSymbolLayer,
-    QgsFontMarkerSymbolLayer, QgsEllipseSymbolLayer, QgsSimpleLineSymbolLayer,
-    QgsMarkerLineSymbolLayer, QgsMarkerSymbol, QgsSimpleFillSymbolLayer, QgsSVGFillSymbolLayer,
-    QgsLinePatternFillSymbolLayer, QgsPointPatternFillSymbolLayer, QgsVectorLayer, QgsVectorLayerSimpleLabeling,
-    QgsTextBufferSettings, QgsPalLayerSettings, QgsTextBackgroundSettings, QgsRuleBasedLabeling)
-from qgis.testing import start_app, unittest
+    Qgis,
+    QgsEllipseSymbolLayer,
+    QgsFillSymbol,
+    QgsFontMarkerSymbolLayer,
+    QgsLinePatternFillSymbolLayer,
+    QgsLineSymbol,
+    QgsMarkerLineSymbolLayer,
+    QgsMarkerSymbol,
+    QgsPalLayerSettings,
+    QgsPointPatternFillSymbolLayer,
+    QgsProperty,
+    QgsRuleBasedLabeling,
+    QgsSimpleFillSymbolLayer,
+    QgsSimpleLineSymbolLayer,
+    QgsSimpleMarkerSymbolLayer,
+    QgsSimpleMarkerSymbolLayerBase,
+    QgsSldExportContext,
+    QgsSVGFillSymbolLayer,
+    QgsSvgMarkerSymbolLayer,
+    QgsSymbolLayer,
+    QgsTextBackgroundSettings,
+    QgsTextBufferSettings,
+    QgsUnitTypes,
+    QgsVectorLayer,
+    QgsVectorLayerSimpleLabeling,
+)
+import unittest
+from qgis.testing import start_app, QgisTestCase
+
 from utilities import unitTestDataPath
 
 # Convenience instances in case you may need them
@@ -39,7 +70,7 @@ from utilities import unitTestDataPath
 start_app()
 
 
-class TestQgsSymbolLayerCreateSld(unittest.TestCase):
+class TestQgsSymbolLayerCreateSld(QgisTestCase):
     """
      This class tests the creation of SLD from QGis layers
      """
@@ -53,6 +84,12 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
         self.assertStaticRotation(root, '50')
 
+        symbol.setAngle(-50)
+        dom, root = self.symbolToSld(symbol)
+        # print( "Simple marker rotation: " + root.ownerDocument().toString())
+
+        self.assertStaticRotation(root, '-50')
+
     def testSimpleMarkerUnitDefault(self):
         symbol = QgsSimpleMarkerSymbolLayer(
             QgsSimpleMarkerSymbolLayerBase.Star, color=QColor(255, 0, 0), strokeColor=QColor(0, 255, 0), size=10)
@@ -68,6 +105,12 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStrokeWidth(root, 2, 11)
         self.assertStaticDisplacement(root, 18, 36)
 
+        # Check negative offset
+        symbol.setOffset(QPointF(-5, -10))
+        dom, root = self.symbolToSld(symbol)
+        # print("Simple marker unit mm: " + root.ownerDocument().toString())
+        self.assertStaticDisplacement(root, -18, -36)
+
     def testSimpleMarkerUnitPixels(self):
         symbol = QgsSimpleMarkerSymbolLayer(
             QgsSimpleMarkerSymbolLayerBase.Star, color=QColor(255, 0, 0), strokeColor=QColor(0, 255, 0), size=10)
@@ -109,6 +152,10 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStaticRotation(root, '90')
         self.assertStaticDisplacement(root, 18, 36)
 
+        symbol.setAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testSvgMarkerUnitPixels(self):
         symbol = QgsSvgMarkerSymbolLayer('symbols/star.svg', 10, 0)
         symbol.setFillColor(QColor("blue"))
@@ -142,6 +189,10 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStaticRotation(root, '45')
         self.assertStaticDisplacement(root, 18, 36)
 
+        symbol.setAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testFontMarkerUnitPixel(self):
         symbol = QgsFontMarkerSymbolLayer('sans', ',', 10, QColor('black'), 45)
         symbol.setOffset(QPointF(5, 10))
@@ -154,11 +205,15 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStaticRotation(root, '45')
         self.assertStaticDisplacement(root, 5, 10)
 
+        symbol.setAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def createEllipseSymbolLayer(self):
         # No way to build it programmatically...
         mTestName = 'QgsEllipseSymbolLayer'
         mFilePath = QDir.toNativeSeparators(
-            '%s/symbol_layer/%s.sld' % (unitTestDataPath(), mTestName))
+            f'{unitTestDataPath()}/symbol_layer/{mTestName}.sld')
 
         mDoc = QDomDocument(mTestName)
         mFile = QFile(mFilePath)
@@ -293,6 +348,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         symbol.setSvgFillColor(QColor('blue'))
         symbol.setSvgStrokeWidth(3)
         symbol.setSvgStrokeColor(QColor('yellow'))
+        symbol.setSubSymbol(QgsLineSymbol())
         symbol.subSymbol().setWidth(10)
 
         dom, root = self.symbolToSld(symbol)
@@ -311,8 +367,13 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         lineSymbolizer = root.elementsByTagName('se:LineSymbolizer').item(0).toElement()
         self.assertStrokeWidth(lineSymbolizer, 1, 36)
 
+        symbol.setAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testSvgFillPixel(self):
         symbol = QgsSVGFillSymbolLayer('test/star.svg', 10, 45)
+        symbol.setSubSymbol(QgsLineSymbol())
         symbol.setSvgFillColor(QColor('blue'))
         symbol.setSvgStrokeWidth(3)
         symbol.setSvgStrokeColor(QColor('black'))
@@ -335,6 +396,10 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         lineSymbolizer = root.elementsByTagName('se:LineSymbolizer').item(0).toElement()
         self.assertStrokeWidth(lineSymbolizer, 1, 10)
 
+        symbol.setAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testLineFillDefault(self):
         symbol = QgsLinePatternFillSymbolLayer()
         symbol.setLineAngle(45)
@@ -349,6 +414,10 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStaticSize(root, '18')
         self.assertStaticDisplacement(root, 15, 9)
 
+        symbol.setLineAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testLineFillPixels(self):
         symbol = QgsLinePatternFillSymbolLayer()
         symbol.setLineAngle(45)
@@ -364,6 +433,10 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         self.assertStaticSize(root, '5')
         self.assertStaticDisplacement(root, 4.25, 2.63)
 
+        symbol.setLineAngle(-45)
+        dom, root = self.symbolToSld(symbol)
+        self.assertStaticRotation(root, '-45')
+
     def testPointFillDefault(self):
         symbol = QgsPointPatternFillSymbolLayer()
         dom, root = self.symbolToSld(symbol)
@@ -381,7 +454,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
     def testSingleSymbolNoScaleDependencies(self):
         layer = QgsVectorLayer("Point", "addfeat", "memory")
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "singleSymbol"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/singleSymbol.qml")
         layer.loadNamedStyle(mFilePath)
 
         dom, root = self.layerToSld(layer)
@@ -391,7 +464,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
     def testSingleSymbolScaleDependencies(self):
         layer = QgsVectorLayer("Point", "addfeat", "memory")
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "singleSymbol"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/singleSymbol.qml")
         layer.loadNamedStyle(mFilePath)
         layer.setMaximumScale(1000)
         layer.setMinimumScale(500000)
@@ -404,7 +477,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
     def testCategorizedNoScaleDependencies(self):
         layer = QgsVectorLayer("Polygon", "addfeat", "memory")
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "categorized"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/categorized.qml")
         layer.loadNamedStyle(mFilePath)
 
         dom, root = self.layerToSld(layer)
@@ -416,7 +489,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
     def testCategorizedWithScaleDependencies(self):
         layer = QgsVectorLayer("Polygon", "addfeat", "memory")
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "categorized"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/categorized.qml")
         layer.loadNamedStyle(mFilePath)
         layer.setMaximumScale(1000)
         layer.setMinimumScale(500000)
@@ -432,7 +505,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
     def testGraduatedNoScaleDependencies(self):
         layer = QgsVectorLayer("Polygon", "addfeat", "memory")
 
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "graduated"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/graduated.qml")
         status = layer.loadNamedStyle(mFilePath)  # NOQA
 
         dom, root = self.layerToSld(layer)
@@ -458,7 +531,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
     def testRuleBasedNoRootScaleDependencies(self):
         layer = QgsVectorLayer("Polygon", "addfeat", "memory")
 
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "ruleBased"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/ruleBased.qml")
         status = layer.loadNamedStyle(mFilePath)  # NOQA
         layer.setMaximumScale(5000)
         layer.setMinimumScale(50000000)
@@ -475,7 +548,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         layer = QgsVectorLayer("Point", "addfeat", "memory")
 
         mFilePath = QDir.toNativeSeparators(
-            '%s/symbol_layer/%s.qml' % (unitTestDataPath(), "categorizedFunctionConflict"))
+            f"{unitTestDataPath()}/symbol_layer/categorizedFunctionConflict.qml")
         status = layer.loadNamedStyle(mFilePath)  # NOQA
 
         dom, root = self.layerToSld(layer)
@@ -800,13 +873,13 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
     def testPointPlacementBelow(self):
         self.assertLabelQuadrant(QgsPalLayerSettings.QuadrantBelow, "Below", 0.5, 1)
 
-    def testPointPlacementAboveRight(self):
+    def testPointPlacementBelowRight(self):
         self.assertLabelQuadrant(QgsPalLayerSettings.QuadrantBelowRight, "BelowRight", 0, 1)
 
-    def testPointPlacementCartoraphic(self):
+    def testPointPlacementCartoraphicOrderedPositionsAroundPoint(self):
         self.assertPointPlacementDistance(QgsPalLayerSettings.OrderedPositionsAroundPoint)
 
-    def testPointPlacementCartoraphic(self):
+    def testPointPlacementCartoraphicAroundPoint(self):
         self.assertPointPlacementDistance(QgsPalLayerSettings.AroundPoint)
 
     def testLineParallelPlacement(self):
@@ -1022,6 +1095,26 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
         else:
             self.assertIsNone(self.assertVendorOption(ts, 'graphic-margin', True))
 
+    def testLabelBackgroundSymbol(self):
+        layer = QgsVectorLayer("Polygon", "addfeat", "memory")
+        self.loadStyleWithCustomProperties(layer, "polygonLabel")
+        settings = layer.labeling().settings()
+        background = QgsTextBackgroundSettings()
+        background.setEnabled(True)
+        background.setType(QgsTextBackgroundSettings.ShapeRectangle)
+        fill_symbol = QgsFillSymbol.createSimple({'color': '#00ffff', 'outline_color': '#00ff00', 'outline_width': 2})
+        background.setFillSymbol(fill_symbol)
+        format = settings.format()
+        format.setBackground(background)
+        settings.setFormat(format)
+        layer.setLabeling(QgsVectorLayerSimpleLabeling(settings))
+
+        dom, root = self.layerToSld(layer)
+
+        ts = self.getTextSymbolizer(root, 1, 0)
+        graphic = self.assertElement(ts, "se:Graphic", 0)
+        self.assertWellKnownMark(graphic, 0, 'square', '#00ffff', '#00ff00', 7)
+
     def testRuleBasedLabels(self):
         layer = QgsVectorLayer("Point", "addfeat", "memory")
         self.loadStyleWithCustomProperties(layer, "ruleLabel")
@@ -1134,7 +1227,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
     def loadStyleWithCustomProperties(self, layer, qmlFileName):
         # load the style, only vector symbology
-        path = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), qmlFileName))
+        path = QDir.toNativeSeparators(f'{unitTestDataPath()}/symbol_layer/{qmlFileName}.qml')
 
         # labeling is in custom properties, they need to be loaded separately
         status = layer.loadNamedStyle(path)
@@ -1209,7 +1302,7 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
     def testRuleBaseEmptyFilter(self):
         layer = QgsVectorLayer("Point", "addfeat", "memory")
 
-        mFilePath = QDir.toNativeSeparators('%s/symbol_layer/%s.qml' % (unitTestDataPath(), "categorizedEmptyValue"))
+        mFilePath = QDir.toNativeSeparators(f"{unitTestDataPath()}/symbol_layer/categorizedEmptyValue.qml")
         status = layer.loadNamedStyle(mFilePath)  # NOQA
 
         dom, root = self.layerToSld(layer)
@@ -1217,11 +1310,79 @@ class TestQgsSymbolLayerCreateSld(unittest.TestCase):
 
         # get the third rule
         rule = root.elementsByTagName('se:Rule').item(2).toElement()
-        filter = rule.elementsByTagName('Filter').item(0).toElement()
-        filter = filter.firstChild().toElement()
-        self.assertEqual("ogc:Or", filter.nodeName())
-        self.assertEqual(1, filter.elementsByTagName('ogc:PropertyIsEqualTo').size())
-        self.assertEqual(1, filter.elementsByTagName('ogc:PropertyIsNull').size())
+        filter = rule.elementsByTagName('ElseFilter').item(0).toElement()
+        self.assertEqual("se:ElseFilter", filter.nodeName())
+
+    def testDataDefinedAngle(self):
+
+        l = QgsSimpleMarkerSymbolLayer(Qgis.MarkerShape.Triangle)
+        p = l.dataDefinedProperties()
+        p.setProperty(QgsSymbolLayer.Property.PropertyAngle, QgsProperty.fromExpression('"field_a"'))
+        dom = QDomDocument()
+        root = dom.createElement("FakeRoot")
+        dom.appendChild(root)
+        l.toSld(dom, root, dict())
+        rot = dom.elementsByTagName('se:Rotation').at(0).firstChild().toElement()
+        self.assertEqual(rot.tagName(), 'ogc:PropertyName')
+        self.assertEqual(rot.text(), 'field_a')
+
+    def testSaveSldStyleV2Png(self):
+        """Test SLD export for polygons with PNG tiles"""
+
+        # Point pattern
+        layer = QgsVectorLayer("Polygon", "addfeat", "memory")
+        error, ok = layer.loadSldStyle(os.path.join(unitTestDataPath('symbol_layer'), 'QgsPointPatternFillSymbolLayer.sld'))
+        self.assertTrue(ok)
+        temp_dir = QTemporaryDir()
+        temp_path = temp_dir.path()
+        sld_path = os.path.join(temp_path, 'export.sld')
+        context = QgsSldExportContext(Qgis.SldExportOption.Png, Qgis.SldExportVendorExtension.NoVendorExtension, sld_path)
+        message, ok = layer.saveSldStyleV2(context)
+        self.assertTrue(ok)
+        self.assertTrue(os.path.exists(os.path.join(temp_path, 'export.png')))
+
+        with open(sld_path) as f:
+            self.assertIn('export.png', f.read())
+
+        image = QImage(os.path.join(temp_path, 'export.png'))
+        self.assertTrue(image.hasAlphaChannel())
+        self.assertEqual(image.height(), 33)
+        self.assertEqual(image.width(), 33)
+
+        for x, y in ((16, 0), (0, 16), (16, 16), (32, 16), (16, 32)):
+            self.assertEqual(image.pixelColor(x, y).name(), '#000000')
+            self.assertEqual(image.pixelColor(x, y).alpha(), 0)
+
+        for x, y in ((0, 0), (0, 32), (32, 0), (32, 32)):
+            self.assertNotEqual(image.pixelColor(x, y).name(), '#000000')
+            self.assertGreater(image.pixelColor(x, y).alpha(), 0)
+
+        # Line pattern
+        error, ok = layer.loadSldStyle(os.path.join(unitTestDataPath('symbol_layer'), 'QgsLinePatternFillSymbolLayer.sld'))
+        self.assertTrue(ok)
+        temp_dir = QTemporaryDir()
+        temp_path = temp_dir.path()
+        sld_path = os.path.join(temp_path, 'export.sld')
+        context = QgsSldExportContext(Qgis.SldExportOption.Png, Qgis.SldExportVendorExtension.NoVendorExtension, sld_path)
+        message, ok = layer.saveSldStyleV2(context)
+        self.assertTrue(ok)
+        self.assertTrue(os.path.exists(os.path.join(temp_path, 'export.png')))
+
+        with open(sld_path) as f:
+            self.assertIn('export.png', f.read())
+
+        image = QImage(os.path.join(temp_path, 'export.png'))
+        self.assertTrue(image.hasAlphaChannel())
+        self.assertEqual(image.height(), 7)
+        self.assertEqual(image.width(), 4)
+
+        for x, y in ((0, 0), (3, 3), (3, 3), (0, 6)):
+            self.assertEqual(image.pixelColor(x, y).name(), '#000000')
+            self.assertEqual(image.pixelColor(x, y).alpha(), 0)
+
+        for x, y in ((2, 0), (0, 3), (3, 6)):
+            self.assertNotEqual(image.pixelColor(x, y).name(), '#000000')
+            self.assertGreater(image.pixelColor(x, y).alpha(), 0)
 
     def assertScaleDenominator(self, root, expectedMinScale, expectedMaxScale, index=0):
         rule = root.elementsByTagName('se:Rule').item(index).toElement()
